// ===  refine_e6d.m ===
//
// Computational Approaches to Lattice
// Packing and Covering Problems
// Achill Schuermann, Frank Vallentin,
// arXiv:math.MG/0403272
//
// written for MAGMA V2.8-3

// 1. compute all non-equivalent Delone polytopes of E6D

e6d := LatticeWithGram(GramMatrix(Dual(Lattice("E",6))));
vertices,_,_ := VoronoiCell(e6d);

vertices_set := Set(vertices);
nonequivalent := {};
while not IsEmpty(vertices_set) do
  v := Rep(vertices_set);
  Include(~nonequivalent, v);
  orbit := {};
  for s in vertices_set do
    if v-s in e6d or -v-s in e6d then Include(~orbit, s); end if;
  end for;
  vertices_set := vertices_set diff orbit;
end while;

// 2. compute for every Delone polytope the 3 possible triangulations

for v in nonequivalent do
  delone := ClosestVectors(e6d, v);
  j := 1;
  triangles := [];
  triangles[1] := [i : i in [1..9] | 
    i eq j or (delone[j]-delone[i],delone[j]-delone[i]) eq 6];
  j := Rep({1..9} diff Set(triangles[1]));
  triangles[2] := [i : i in [1..9] | 
    i eq j or (delone[j]-delone[i],delone[j]-delone[i]) eq 6];
  triangles[3] := [i : i in [1..9] | 
    not i in triangles[1] and not i in triangles[2]];

  for i := 1 to 3 do
    j := i mod 3 + 1;
    k := (i + 1) mod 3 + 1;
    for u := 1 to 3 do
      for v := 1 to 3 do
        simplex := 
          {delone[m] : m in {1..9} diff {triangles[j][u], triangles[k][v]}};
	if not e6d ! 0 in simplex then
          vertex := Rep(simplex);
	  simplex := {v - vertex : v in simplex};   
        end if;
        for vertex in simplex do
          if not vertex eq e6d ! 0 then
            for c in [1..6] do printf "%o ", vertex[c]; end for;
            printf "\n";
          end if;
        end for;
      end for;
    end for;
  end for;
end for;

