/* -*- Mode:C++; -*- */

// QQ.h

#ifndef __QQ_H
#define __QQ_H


#include <iostream>
#include <NTL/ZZ.h>


// Be aware! It is not checked if denominator is 0. [+-1/0 = +-infinity]

class QQ {

public:
  QQ() { num=0; den=1; } // initial value is 0
  QQ(const QQ& r) { num=r.num; den=r.den; }
  QQ(const NTL::ZZ& n, const NTL::ZZ& d) { NTL::ZZ g; GCD(g,n,d); div(num,n,g); div(den,d,g); if (den<0) { num=-num; den=-den; } } // always reduced with den>=0
  QQ(const long n, const long d) { NTL::ZZ g; GCD(g,NTL::to_ZZ(n),NTL::to_ZZ(d)); div(num,NTL::to_ZZ(n),g); div(den,NTL::to_ZZ(d),g); if (den<0) { num=-num; den=-den; } }
  QQ(const NTL::ZZ& n) { num=NTL::to_ZZ(n); den=NTL::to_ZZ(1); }
  QQ(const long n) { num=NTL::to_ZZ(n); den=NTL::to_ZZ(1); }

  QQ& operator=(const QQ& r) { num=r.num; den=r.den; return *this; }

  friend NTL::ZZ Num(const QQ& r) { return r.num; }
  friend NTL::ZZ Den(const QQ& r) { return r.den; }

  friend QQ to_QQ(const int n, const int d) { return QQ(NTL::to_ZZ(n),NTL::to_ZZ(d)); }
  friend QQ to_QQ(const long n, const long d) { return QQ(NTL::to_ZZ(n),NTL::to_ZZ(d)); }
  friend QQ to_QQ(const char* n, const char* d) { return QQ(NTL::to_ZZ(n),NTL::to_ZZ(d)); }

  friend QQ to_QQ(const int n) { return QQ(NTL::to_ZZ(n),NTL::to_ZZ(1)); }
  friend QQ to_QQ(const long n) { return QQ(NTL::to_ZZ(n),NTL::to_ZZ(1)); }
  
  friend double to_double(const QQ& r) { return NTL::to_double(r.num)/NTL::to_double(r.den); }

  QQ operator+(const QQ& r) const { return QQ(num*r.den+r.num*den,den*r.den); }
  QQ operator-(const QQ& r) const { return QQ(num*r.den-r.num*den,den*r.den); }
  QQ operator*(const QQ& r) const { return QQ(num*r.num,den*r.den); }
  QQ operator/(const QQ& r) const { return QQ(num*r.den,den*r.num); }

  QQ operator+(const long r) const { return QQ(num+r*den,den*r); }
  QQ operator-(const long r) const { return QQ(num-r*den,den*r); }
  QQ operator*(const long r) const { return QQ(num*r,den); }
  QQ operator/(const long r) const { return QQ(num,den*r); }

  QQ operator-() { return QQ(-num,den); }
  friend QQ abs(const QQ& r) { return QQ(abs(r.num),r.den); }

  bool operator==(const QQ& r) { return (num==r.num) && (den==r.den); }
  bool operator!=(const QQ& r) { return (num!=r.num) || (den!=r.den); }
  bool operator<(const QQ& r) { return (num*r.den < r.num*den); }
  bool operator>(const QQ& r) { return (num*r.den > r.num*den); }
  bool operator<=(const QQ& r) { return (num*r.den <= r.num*den); }
  bool operator>=(const QQ& r) { return (num*r.den >= r.num*den); }

  bool operator==(const long r) { return (num==r) && (den==1); }
  bool operator!=(const long r) { return (num!=r) || (den!=1); }
  bool operator<(const long r) { return (num < r*den); }
  bool operator>(const long r) { return (num > r*den); }
  bool operator<=(const long r) { return (num <= r*den); }
  bool operator>=(const long r) { return (num >= r*den); }

  friend std::ostream& operator<<(std::ostream& out, const QQ& r) {
    if (r.den==1) 
      out << r.num;
    else
      out << r.num << "/" << r.den;
    return out;
  }

private:	
  
  NTL::ZZ num; 
  NTL::ZZ den;         
};


#endif //__QQ_H

