/* -*- Mode:C++; -*- */

// e6d.cc

#include <math.h>
#include <iostream>
#include <vector>
#include <NTL/mat_ZZ.h>

#include "covering.h"

//
//
//

void die_e6d(char* last_words)
{
  std::cout <<"e6d.cc: " <<last_words;
  exit(EXIT_FAILURE);
}

//
//
//

void branch_and_bound
(
int depth,
int* e6d_simplices,
std::vector<int>& chosen_tria
)
{
  int dim = 6;

  if (depth == 40) return;

  std::cout <<"* depth = " <<depth <<": ";

  for (int i = 0; i <= depth; i++)
    std::cout <<chosen_tria[i] <<" ";
  std::cout <<"\n";
  
  int simplex_count = (depth+1)*9;
  int* simplex;
  simplex = (int*) calloc(simplex_count*dim*dim, sizeof(int));
  if (simplex == NULL) die_e6d("simplex == NULL.\n");
  for (int choice = 0; choice <= depth; choice++)
    for (int simplex_index = 0; simplex_index < 9; simplex_index++)
      for (int vertex = 0; vertex < 6; vertex++)
	for (int coord = 0; coord < 6; coord++) 
	  simplex[choice*9*6*6 + simplex_index*6*6 + vertex*6 + coord] = 
	    e6d_simplices[choice*3*9*6*6 + chosen_tria[choice]*9*6*6 + simplex_index*6*6 + vertex*6 + coord];
  
  int basis_count;
  basis_count = dim*(dim+1)/2;
  int* basis;
  basis = (int*) calloc(basis_count*dim*(dim+1)/2, sizeof(int));
  if (basis == NULL) die_e6d("basis == NULL.\n");
  for (int basis_index = 0; basis_index < basis_count; basis_index++) 
    basis[basis_index*dim*(dim+1)/2 + basis_index] = 1;
  
  int inequality_count = 0;
  int* inequality = NULL;

  maxdet_data md;
  
  setup_maxdet_covering_constraints(dim, simplex_count, simplex, basis_count, basis, inequality_count, inequality, md);
  setup_maxdet_covering_cost_func(dim, basis_count, basis, md);

  double* interior;
  interior = (double*) calloc(basis_count, sizeof(double));
  interior[ 0] = 0.4;
  interior[ 1] = 0.1;  interior[ 2] = 0.4;  
  interior[ 3] = 0.2;  interior[ 4] = 0.2;  interior[ 5] = 0.4;  
  interior[ 6] = 0.2;  interior[ 7] = 0.2;  interior[ 8] = 0.1;  interior[ 9] = 0.4; 
  interior[10] =-0.1;  interior[11] = 0.2;  interior[12] = 0.1;  interior[13] = 0.1;  interior[14] = 0.4;  
  interior[15] = 0.1;  interior[16] = 0.1;  interior[17] = 0.2;  interior[18] = 0.2;  interior[19] = 0.2;  interior[20] = 0.4;

  int max_iterations = 100;

  double duality_gap = 1e-06;

  double lower_bound;
  double upper_bound;
  solve_maxdet_nonrigorously(md, interior, duality_gap, max_iterations,
			     lower_bound, upper_bound, OUTPUT_LEVEL_NONE);

  std::cout <<"  non rigorous upper bound: " <<1/sqrt(exp(-upper_bound)) <<"\n";

  bool stop_here = false;

  if ((1/sqrt(exp(-upper_bound))) > 0.477) {
    std::cout <<"  check it rigorously..." <<std::flush;
    NTL::vec_ZZ minimizer_approx;
    NTL::vec_ZZ dual_Z_approx;       
    NTL::vec_ZZ dual_W_approx;       
    maxdet_bound lower_bound;
    maxdet_bound upper_bound;

    maxdet_bound q6c1_bound;
    QQ det_part(227529, 1000000); // this corresponds to 0.4797 _exact_
    q6c1_bound.det_part = det_part;
    q6c1_bound.lin_part = 0;

    bool rigorous = 
      solve_maxdet_rigorously(md, interior, duality_gap, max_iterations, minimizer_approx, 
			      dual_Z_approx, dual_W_approx, lower_bound, upper_bound, OUTPUT_LEVEL_UNCOMMUNICATIVE);

    if (!rigorous) {
      std::cout <<"can't do it, go on\n";
    }
    if (rigorous && upper_bound > q6c1_bound) {
      std::cout <<"bound reached, stop here\n";
      stop_here = true;      
    }
    else {
      std::cout <<"bound not reached, go on\n";
    }
  }
    
  // free me

  free_maxdet_covering_data(md);  
  free(interior);
  free(basis);
  free(simplex);

  if (!stop_here) {
    chosen_tria[depth+1] = 0;
    branch_and_bound(depth+1, e6d_simplices, chosen_tria);
    chosen_tria[depth+1] = 1;
    branch_and_bound(depth+1, e6d_simplices, chosen_tria);
    chosen_tria[depth+1] = 2;
    branch_and_bound(depth+1, e6d_simplices, chosen_tria);
  }
}

//
//
//

int main() 
{
  int* e6d_simplices;
  e6d_simplices = (int*) calloc(40*3*9*6*6, sizeof(int));
  if (e6d_simplices == NULL) die_e6d("e6d_simplices == NULL.\n");

  for (int i = 0; i < 40*3*9*6*6; i++)
    std::cin >>e6d_simplices[i];
  
  std::vector<int> choice(40);
  
  choice[0] = 0;
  branch_and_bound(0, e6d_simplices, choice);
  choice[0] = 1;
  branch_and_bound(0, e6d_simplices, choice);
  choice[0] = 2;
  branch_and_bound(0, e6d_simplices, choice);
}

